' AdvanceLcd
' Copyright  Christian Klaussner
' 
' advancelcd.bmx

Strict

Rem
bbdoc: AdvanceLcd
about: This module provides the ability to write applets for Logitech LCD devices.

<b>Information</b><br />
You don't need real hardware to develop applets. Just right-click on the LCD Manager icon in the system tray 
while holding down Ctrl+Shift to open the extended menu where you can start emulators for both monochrome and 
color devices.

<b>License</b><br />
This work is licensed under the Creative Commons Attribution-No Derivative Works 3.0 United States License. 
To view a copy of this license, visit http://creativecommons.org/licenses/by-nd/3.0/us/ or send a letter to 
Creative Commons, 171 Second Street, Suite 300, San Francisco, California, 94105, USA.
End Rem
Module chris.advancelcd

?Win32

Import "advlcd.lib"
Include "native.bmx"
Include "constants.bmx"

Rem
bbdoc: Represents an applet that runs on an LCD device.
End Rem
Type TLcdApplet
	Field applet:Byte Ptr
	Field handler:Int(sender:TLcdApplet, event:Int, data:Int)
	
	Function PrivateEventHandler(event:Int, context:Object, data:Int) "Win32"
		Local applet:TLcdApplet = TLcdApplet(context)
		
		If applet.handler applet.handler(applet, event, data)
	End Function
	
	Rem
	bbdoc: Creates a new applet.
	about: Parameter @devices specifies the device types and can be a combination of the following values.
	
	<table>
	<tr><td><b>Device type</b></td><td><b>Description</b></td></tr>
	<tr><td>LCD_DEVICE_MONO</td><td>Support for monochrome displays (G13, G15 and Z-10)</td></tr>
	<tr><td>LCD_DEVICE_COLOR</td><td>Support for color displays (G19)</td></tr>
	</table>
	
	If the applet supports both device types, all information (e.g. current color, font, render target, etc.) 
	is stored separately for each device. Use the #SelectDevice method to select the target device for 
	all subsequent device-specific operations.
	Methods that affect only the current device are marked as [device-specific] in this documentation.
	
	If @configurable is True, the 'Configure...' button in the LCD Manager will be enabled and the applet will 
	receive an LCD_EVENT_CONFIGURE event through its event handler when the user clicks on in.
	If @autostart is True, the applet can automatically be launched by the LCD Manager, otherwise the user 
	has to start it manually.
	
	Throws a #TLcdException if the applet could not be created.
	You should delete the applet with its #Dispose method if it is no longer needed and before you call
	#LcdDeInitialize.
	End Rem
	Method Init:TLcdApplet(name:String, devices:Int, configurable:Int, autostart:Int)
		applet = LcdCreateApplet(name, devices, configurable, autostart)
		If Not applet Throw New TLcdException.Init("Unable to create applet")
		
		LcdSetEventHandler(applet, PrivateEventHandler, Self)
		
		Return Self
	End Method
	
	Rem
	bbdoc: Deletes the applet.
	about: You should delete all applets before you call #LcdDeInitialize.
	End Rem
	Method Dispose()
		LcdDeleteApplet(applet)
		applet = Null
	End Method
	
	Method Delete()
		If applet LcdDeleteApplet(applet)
	End Method
	
	Rem
	bbdoc: Checks if the applet is currently enabled.
	returns: True if the applet is enabled in the LCD Manager, otherwise False.
	End Rem
	Method Enabled()
		Return LcdAppletEnabled(applet)
	End Method
	
	Rem
	bbdoc: Checks if the applet is opened on the currently selected device.
	returns: True if the applet is opened on the device, otherwise False.
	about: Use #SelectDevice to specify the current device.
	
	[This method is device-specific.]
	End Rem
	Method Opened()
		Return LcdAppletOpened(applet)
	End Method
	
	Rem
	bbdoc: Checks if the applet is active.
	returns: True if the applet is both enabled and opened, otherwise False or LCD_DEVICE_INVALID if the LCD Manager is no longer avaliable.
	about: You should suspend rendering until the applet is active. You can also use #WaitActivation to block the 
	calling thread until the applet gets activated.
	
	[This method is device-specific.]
	End Rem
	Method Active()
		Return LcdAppletActive(applet)
	End Method
	
	Rem
	bbdoc: Waits until the applet gets activated.
	returns: The devices that are currently active or LCD_DEVICE_INVALID if the LCD Manager is no longer avaliable.
	about: If the return value is LCD_DEVICE_INVALID the connection to the LCD Manager is lost and cannot be restored. 
	In this case, the applet should stop all of its LCD-related functionality or exit.
	End Rem
	Method WaitActivation()
		Return LcdWaitActivation(applet)
	End Method
	
	Rem
	bbdoc: Determines the target of subsequent, device-specific operations.
	about: You only need to call this method if the applet supports both monochrome and color devices. 
	If only one device type is supported the device will be selected automatically.
	End Rem
	Method SelectDevice(device:Int)
		Return LcdSelectDevice(applet, device)
	End Method
	
	Rem
	bbdoc: Sets the event handler for configuration and button events.
	about: The event type is indicated by the @event parameter which can be one of the following values.
	
	<table>
	<tr><td><b>Event type</b></td><td><b>Description</b></td></tr>
	<tr><td>LCD_EVENT_CONFIGURE</td><td>The user pressed the 'Configure...' button in the LCD Manager.</td></tr>
	
	<tr><td>LCD_EVENT_BUTTON</td><td>The user pressed or released a soft button. 
	Parameter @data contains a bitmask of buttons. See @constants.bmx for valid button identifiers.</td></tr>
	</table>
	
	Note that the event handler is executed in the context of the LCD Manager and is therefore running 
	in a different thread. You need to build in 'threaded mode' and ensure thread safety if the event handler 
	shares resources with threads in your application.
	End Rem
	Method SetEventHandler(eventhandler:Int(sender:TLcdApplet, event:Int, data:Int))
		handler = eventhandler
	End Method
	
	Rem
	bbdoc: Determines the button status.
	returns: True if @button is currently pressed, otherwise False.
	about: You can also use #SetEventHandler for asynchronous event handling. See @constants.bmx for valid 
	button identifiers.
	
	[This method is device-specific.]
	End Rem
	Method ButtonState(button:Int)
		Return LcdButtonState(applet, button)
	End Method
	
	Rem
	bbdoc: Determines the configuration status.
	returns: True if the user pressed the 'Configure...' button in the LCD Manager since the last call of this method, otherwise False.
	about: You can also use #SetEventHandler for asynchronous event handling.
	End Rem
	Method ConfigState()
		Return LcdConfigState(applet)
	End Method
	
	Rem
	bbdoc: Sets the applets' priority and foreground status.
	about: Parameter @priority can be one of the following values.
	
	<table>
	<tr><td>LCD_PRIORITY_LOW</td></tr>
	<tr><td>LCD_PRIORITY_NORMAL</td></tr>
	<tr><td>LCD_PRIORITY_HIGH</td></tr>
	</table>
	
	Specify True for @foreground to stop the LCD Manager from automatically switching between all 
	running applets or False to continue with automatic switching.
	Note that an applet can easily lose its foreground status when the user manually switches between 
	different applets or returns to the applet menu.
	End Rem
	Method SetPriority(priority:Int, foreground:Int)
		Return LcdSetPriority(applet, priority, foreground)
	End Method
	
	Rem
	bbdoc: Shows the content of the back-buffer render target on the LCD.
	about: Note that the applet only becomes visible after the first call to this method. That way you can 
	do applet-specific initializations without the applet being displayed on the device.
	If @sync is True this method will return immediately after sending the buffer data to the LCD Manager, 
	otherwise it will wait until the data is actually displayed which takes about 30 milliseconds or more.	
	
	[This method is device-specific.]
	End Rem
	Method Update(sync:Int = False)
		Return LcdUpdate(applet, sync)
	End Method
	
	Rem
	bbdoc: Draws a pixmap.
	about: Parameter @frame specifies the frame number if the pixmap was created with the @TLcdPixmap.FromAnimFile 
	method.
	
	[This method is device-specific.]
	End Rem
	Method DrawPixmap(pixmap:TLcdPixmap, x:Int, y:Int, frame:Int = 0)
		If Not pixmap Return LCD_ERROR
		
		Return LcdDrawPixmap(applet, pixmap.pixmap, x, y, frame)
	End Method
	
	Rem
	bbdoc: Draws a rectangular area of a pixmap.
	about: Parameters @sx, @sy, @width and @height specify the source rectangle of @pixmap to be drawn.
	
	[This method is device-specific.]
	End Rem
	Method DrawPixmapRect(pixmap:TLcdPixmap, x:Int, y:Int, sx:Int, sy:Int, width:Int, height:Int, frame:Int = 0)
		If Not pixmap Return 1
		
		Return LcdDrawPixmapRect(applet, pixmap.pixmap, x, y, sx, sy, width, height, frame)
	End Method
	
	Rem
	bbdoc: Sets the current render target.
	about: Parameter @pixmap specifies the pixmap that will be used as render target.
	Note that only pixmaps created by @TPixmap.Init are suitable render targets. Drawing to 
	pixmaps that are loaded from image files is not supported.
	Specify Null for @pixmap to reset the render target.
	
	[This method is device-specific.]
	End Rem
	Method SetTarget(pixmap:TLcdPixmap)
		Local nativepixmap:Byte Ptr
		If pixmap nativepixmap = pixmap.pixmap Else nativepixmap = Null
		
		Return LcdSetTarget(applet, nativepixmap)
	End Method
	
	Rem
	bbdoc: Enables or disables anti-aliasing.
	about: Note that anti-aliasing is only avaliable for color devices.
	
	[This method is device-specific.]
	End Rem
	Method EnableAntiAlias(enable:Int)
		Return LcdEnableAntiAlias(applet, enable)
	End Method
	
	Rem
	bbdoc: Deletes the content of the current render target.
	about: Parameter @color specifies the color in RGB format. Note that any alpha component of 
	the color will be ignored.
	
	[This method is device-specific.]
	End Rem
	Method Clear(color:Int)
		Return LcdClear(applet, color)
	End Method
	
	Rem
	bbdoc: Sets the current drawing color.
	about: Parameter @color specifies the color in RGB format. Note that any alpha component of 
	the color will be ignored. 
	You can also use the following color constants for monochrome displays.
	
	<table>
	<tr><td><b>Constant</b></td><td><b>Description</b></td></tr>
	<tr><td>LCD_COLOR_VISIBLE</td><td>Makes pixels visible (similar to white on color displays).</td></tr>
	<tr><td>LCD_COLOR_INVISIBLE</td><td>Makes pixels invisible (similar to black on color displays).</td></tr>
	</table>
	
	The default color is white or LCD_COLOR_VISIBLE.
	
	[This method is device-specific.]
	End Rem
	Method SetColor(color:Int)
		Return LcdSetColor(applet, color)
	End Method
	
	Rem
	bbdoc: Sets the current line width.
	about: The line width affects the #DrawLine method as well as #DrawRect and #DrawOval if 
	the @fill parameter is False.
	
	[This method is device-specific.]
	End Rem
	Method SetLineWidth:Float(width:Float)
		Return LcdSetLineWidth(applet, width)
	End Method
	
	Rem
	bbdoc: Sets the current rotation.
	about: You can also specify the center of the rotation using @x and @y.
	This method affects all subsequent calls to #DrawLine, #DrawRect and #DrawOval. Note that pixmaps 
	and text cannot be rotated.
	
	[This method is device-specific.]
	End Rem
	Method SetRotation:Float(rotation:Float, x:Int = 0, y:Int = 0)
		Return LcdSetRotation(applet, rotation, x, y)
	End Method
	
	Rem
	bbdoc: Draws a line.
	about: [This method is device-specific.]
	End Rem
	Method DrawLine(x1:Int, y1:Int, x2:Int, y2:Int)
		Return LcdDrawLine(applet, x1, y1, x2, y2)
	End Method
	
	Rem
	bbdoc: Draws a rectangle.
	about: Set @fill to False in order to draw only the outline of the rectangle.
	
	[This method is device-specific.]
	End Rem
	Method DrawRect(x:Int, y:Int, width:Int, height:Int, fill:Int = True)
		Return LcdDrawRect(applet, x, y, width, height, fill)
	End Method
	
	Rem
	bbdoc: Draws an oval.
	about: Set @fill to False in order to draw only the outline of the oval.
	
	[This method is device-specific.]
	End Rem
	Method DrawOval(x:Int, y:Int, width:Int, height:Int, fill:Int = True)
		Return LcdDrawOval(applet, x, y, width, height, fill)
	End Method
	
	Rem
	bbdoc: Sets the current font.
	about: This method affects all subsequent calls to #DrawText.
	
	[This method is device-specific.]
	End Rem
	Method SetFont(font:TLcdFont)
		Local nativefont:Byte Ptr
		If font nativefont = font.font Else nativefont = Null
		
		Return LcdSetFont(applet, nativefont)
	End Method
	
	Rem
	bbdoc: Draws a text string.
	about: [This method is device-specific.]
	End Rem
	Method DrawText(text:String, x:Int, y:Int)
		Return LcdDrawText(applet, text, x, y)
	End Method
	
	Rem
	bbdoc: Calculates the width of a string.
	about: This method uses the currently selected font to measure the string's width.
	
	[This method is device-specific.]
	End Rem
	Method StringWidth(text:String)
		Return LcdStringWidth(applet, text)
	End Method
	
	Rem
	bbdoc: Calculates the height of a string.
	about: This method uses the currently selected font to measure the string's height.
	
	[This method is device-specific.]
	End Rem
	Method StringHeight(text:String)
		Return LcdStringHeight(applet, text)
	End Method
End Type

Rem
bbdoc: Represents a pixmap loaded from a file or created programmatically.
End Rem
Type TLcdPixmap
	Field pixmap:Byte Ptr
	
	Rem
	bbdoc: Creates a new render target pixmap.
	about: Parameter @device indicates the device type of the pixmap. You should use the pixmap only with 
	the device type that it was created for. See @TLcdApplet.Init for valid device types.
	Throws a #TLcdException if the pixmap could not be created. 
	You should delete the pixmap with its #Dispose method if it is no longer needed and before you call 
	#LcdDeInitialize.
	End Rem
	Method Init:TLcdPixmap(width:Int, height:Int, device:Int)
		pixmap = LcdCreatePixmap(width, height, device)
		If Not pixmap Throw New TLcdException.Init("Unable to create pixmap")
		
		Return Self
	End Method
	
	Rem
	bbdoc: Loads a pixmap from an image file.
	about: Supported file formats are PNG, JPEG, BMP, TIFF and GIF.
	
	Throws a #TLcdException if the pixmap could not be loaded. 
	You should delete the pixmap with its #Dispose method if it is no longer needed and before you call 
	#LcdDeInitialize.
	End Rem
	Method FromFile:TLcdPixmap(file:String)
		pixmap = LcdLoadPixmap(file)
		If Not pixmap Throw New TLcdException.Init("Unable to load pixmap")
		
		Return Self
	End Method
	
	Rem
	bbdoc: Loads an animated pixmap from an image file.
	about: Supported file formats are PNG, JPEG, BMP, TIFF, GIF.
	
	Parameters @framewidth and @frameheight indicate the size of a single frame in the pixmap. 
	The pixmap has to be @framewidth * @frame pixels wide and @frameheight pixels high, because the size of 
	all frames has to be equal.
	
	Throws a #TLcdException if the pixmap could not be loaded. 
	You should delete the pixmap with its #Dispose method if it is no longer needed and before you call 
	#LcdDeInitialize.
	End Rem
	Method FromAnimFile:TLcdPixmap(file:String, framewidth:Int, frameheight:Int, frames:Int)
		pixmap = LcdLoadAnimPixmap(file, framewidth, frameheight, frames)
		If Not pixmap Throw New TLcdException.Init("Unabel to load animated pixmap")
		
		Return Self
	End Method
	
	Rem
	bbdoc: Deletes the pixmap.
	about: You should delete all pixmaps before you call #LcdDeInitialize.
	End Rem
	Method Dispose()
		LcdDeletePixmap(pixmap)
		pixmap = Null
	End Method
	
	Method Delete()
		If pixmap LcdDeletePixmap(pixmap)
	End Method
	
	Rem
	bbdoc: Returns the width of the pixmap.
	End Rem
	Method Width()
		Return LcdPixmapWidth(pixmap)
	End Method
	
	Rem
	bbdoc: Returns the height of the pixmap.
	End Rem
	Method Height()
		Return LcdPixmapHeight(pixmap)
	End Method
End Type

Rem
bbdoc: Represents a font loaded from the system or a file.
End Rem
Type TLcdFont
	Field font:Byte Ptr
	
	Rem
	bbdoc: Loads a font from the system or a font file.
	about: The font is loaded from the system by default. If you want to load it from a TTF or OTF file, specify 
	the relative path and filename for parameter @file.
	
	Parameter @flags can be LCD_FONT_DEFAULT or a combination of the following values.
	
	<table>
	<tr><td>LCD_FONT_BOLD</td></tr>
	<tr><td>LCD_FONT_ITALIC</td></tr>
	<tr><td>LCD_FONT_UNDERLINE</td></tr>
	</table>
	
	Throws a #TLcdException if the font could not be loaded.
	You should delete the font with its #Dispose method if it is no longer needed and before you call 
	#LcdDeInitialize.
	End Rem
	Method Init:TLcdFont(family:String, height:Int, flags:Int = LCD_FONT_DEFAULT, file:String = Null)
		font = LcdLoadFont(family, height, flags, file)
		If Not font Throw New TLcdException.Init("Unable to load font")
		
		Return Self
	End Method
	
	Rem
	bbdoc: Deletes the font.
	about: You should delete all fonts before you call #LcdDeInitialize.
	End Rem
	Method Dispose()
		LcdDeleteFont(font)
		font = Null
	End Method
	
	Method Delete()
		If font LcdDeleteFont(font)
	End Method
End Type

Rem
bbdoc: Represents an exception thrown by an AdvanceLcd function.
End Rem
Type TLcdException Extends TBlitzException
	Field message:String
	
	Method Init:TLcdException(msg:String)
		message = msg
		
		Return Self
	End Method
	
	Method ToString:String()
		Return message
	End Method
End Type

?
